/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <DataTypes/DxcException.hpp>
#include <DataTypes/SystemCatalog.hpp>
#include <DxcConfig.hpp>

using namespace Dxc;
using namespace std;

map<string, const SystemCatalog*>* SystemCatalog::myMap_ =
    new map<string, const SystemCatalog*>();

const SystemCatalog* SystemCatalog::getInstance(const string& system)
{
    map<string, const SystemCatalog*>::const_iterator i;
    i = myMap_->find(system);

    if (i != myMap_->end())
    {
        return i->second;
    }
    else
    {
        (*myMap_)[system] = new SystemCatalog(system);
        return (*myMap_)[system];
    }
}

const string SystemCatalog::getSystemName() const
{
    return name_;
}

const set<string>* SystemCatalog::getComponentIDs() const
{
    return &componentIDs_;
}

const set<string>* SystemCatalog::getFaultModeIDs(const std::string& componentID) const
{
    map<string, set<string> >::const_iterator i;
    i = componentToModeIDs_.find(componentID);

    if (i != componentToModeIDs_.end())
        return &(i->second);
    else
        return NULL;
}

const map<string, double>* SystemCatalog::getSensorsAndCosts() const
{
    return &sensorCosts_;
}

bool SystemCatalog::isComponentValid(const string& componentID) const
{
    set<string>::const_iterator i = componentIDs_.find(componentID);

    if (i != componentIDs_.end())
        return true;

    return false;
}

bool SystemCatalog::isFaultModeValid(const string& componentID, const string& modeID) const
{
    const set<string>* modes = NULL;
    map<string, set<string> >::const_iterator i;
    i = componentToModeIDs_.find(componentID);

    if (i != componentToModeIDs_.end())
    {
        modes = &(i->second);
        set<string>::const_iterator j = modes->find(modeID);

        if (j != modes->end())
            return true;
    }

    return false;
}

SystemCatalog::SystemCatalog(const string& system)
{
    name_ = system;
    DxcConfig* cfg = DxcConfig::getInstance();
    string file = cfg->get("SYSTEMCAT_DIR") + DIR_DELIM + system + ".xml";

    TiXmlDocument tXml(file);

    if (!tXml.LoadFile())
        throw DxcException(file + ": unable to open system catalog file.");

    TiXmlElement* nd = tXml.FirstChildElement("systemCatalog");
    nd = firstChild(nd, "systems");
    nd = firstChild(nd, "system");
    nd = firstChild(nd, "components");
    nd = firstChild(nd, "component");

    while (nd)
    {
        string cid = firstChild(nd, "name")->FirstChild()->Value();
        double pcost = 0;
        try
        {
            istringstream pcoststr(
                firstChild(nd, "probeCost")->FirstChild()->Value());
            pcoststr >> pcost;
        }
        catch (...)
        {
            pcost = 0;
        }

        componentIDs_.insert(cid);
        sensorCosts_[cid] = pcost;
        componentToModeIDs_.insert(make_pair(cid, set<string>()));

        string ctype = firstChild(nd, "componentType")->FirstChild()->Value();
        TiXmlElement* mnd = tXml.FirstChildElement("systemCatalog");
        mnd = firstChild(mnd, "componentTypeCatalog");
        mnd = firstChild(mnd, "componentType");

        mnd = findSiblingByValue(mnd, "name", ctype);
        mnd = firstChild(mnd, "modesRef");

        while (mnd)
        {
            string modesRef = mnd->FirstChild()->Value();
            TiXmlElement* fnd = tXml.FirstChildElement("systemCatalog");
            fnd = firstChild(fnd, "modeCatalog");
            fnd = firstChild(fnd, "modeGroup");
            fnd = findSiblingByValue(fnd, "name", modesRef);
            fnd = fnd->FirstChildElement("mode");
            while (fnd)
            {
                if ( ((string)fnd->Attribute("xsi:type")) == "faultMode")
                {
                    string mdName =
                        fnd->FirstChild("name")->FirstChild()->Value();
                    componentToModeIDs_[cid].insert(mdName);
                }

                fnd = fnd->NextSiblingElement("mode");
            }
            mnd = mnd->NextSiblingElement("modesRef");
        }

        nd = nd->NextSiblingElement();
    }
}

TiXmlElement* SystemCatalog::findSiblingByValue(TiXmlElement* node,
        const string& elementName,
        const string& value)
{
    while (node)
    {
        if (node->FirstChild(elementName)->FirstChild()->Value() == value)
            break;

        node = node->NextSiblingElement();
    }

    return node;
}

TiXmlElement* SystemCatalog::firstChild(TiXmlElement* node,
                                        const string& elementName)
{
    if (node)
    {
        node = node->FirstChildElement(elementName);
        if (node)
            return node;
        else
            throw DxcException("Access error in xml catalog. " +
                               elementName + " not found.");
    }
    else
    {
        throw DxcException("Access error in xml catalog.");
    }

    return NULL;
}
