/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef VALUE_HPP
#define VALUE_HPP

#include <iostream>
#include <string>

#include <DxcDefs.hpp>

namespace Dxc
{

/**
 * @brief Abstract interface for values that can be of various concrete datatypes.
 *
 * Value is the superclass of all the basic datatypes that can be referred to by
 * DxcData, such as boolean, integer, real, or string. The abstraction is provided
 * so that data of various kinds can be supplied in a uniform way, eg. for sensor
 * values and command values.
 */
class DXCAPI_EXPORT Value
{
public:
    Value();

public:
    /**
     * @brief Print the value to an output stream.
     */
    virtual std::ostream &put(std::ostream &) const = 0;

    /**
     * @brief Return a copy of the Value
     */
    virtual Value* clone() const = 0;

    bool lt(const Value*) const;
    bool eq(const Value*) const;

    static Value* v(const std::string&);
    static Value* v(const char*);
    static Value* v(int);
    static Value* v(double);
    static Value* v(bool);
};

/**
 * @brief Boolean Value type.
 */
class DXCAPI_EXPORT BoolValue : public Value
{
public:
    BoolValue(bool);
    bool get() const;
    virtual std::ostream &put(std::ostream &) const;
    Value* clone() const;

private:
    bool value_;
};

/**
 * @brief Real (double) Value type.
 */
class DXCAPI_EXPORT RealValue : public Value
{
public:
    double get() const;
    RealValue(double);
    virtual std::ostream &put(std::ostream &) const;
    Value* clone() const;

private:
    double value_;
};

/**
 * @brief Integer (int) Value type.
 */
class DXCAPI_EXPORT IntegerValue : public Value
{
public:
    IntegerValue(int);
    int get() const;
    virtual std::ostream &put(std::ostream &) const;
    Value* clone() const;

private:
    int value_;
};

/**
 * @brief String (STL std::string) Value type.
 */
class DXCAPI_EXPORT StringValue : public Value
{
public:
    StringValue(const std::string *);
    StringValue(const std::string &);
    StringValue(const char *);
    std::string get() const;
    virtual std::ostream &put(std::ostream &) const;
    Value* clone() const;

private:
    const std::string value_;
};

}

std::ostream DXCAPI_EXPORT & operator << (std::ostream &, const Dxc::Value *);

#endif
