/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api;

import org.dxc.api.datatypes.DxcData;

/**
 * The abstract definition of the DcxData callback class.
 * 
 * This abstract class forms the base class for the DXC data callback. Users
 * must extend this abstract class to give their own callback handler, and
 * register the class with a DxcConnector via a ConnectorFactory. Only one
 * callback can be registered per connector, but the same callback can be
 * registered with different connectors. When new data is received by a
 * connector, the handler function will be called in the context of a new
 * thread. The DxcConnector stores a reference to the instance of the callback
 * class; users are responsible for ensuring that the instance is not destroyed
 * before the connector stops receiving data.
 */
public abstract class DxcCallback {

    /**
     * Synchronized callback method.
     * 
     * Do not override this method unless you know what you're doing.
     * 
     * @param data
     *            the message to be processed by the processData function
     */
    public synchronized void onDataReceived(DxcData data) {
        processData(data);
    }

    /**
     * Method to be called when data is received.
     * 
     * Implement this method to receive DxcData messages. Each message must be
     * downcast into its specific type.
     * 
     * @param data
     *            the message to be processed by the callback function
     */
    public abstract void processData(DxcData data);

}
