package org.dxc.api.datatypes;

import java.util.Map;

/**
 * DXC Data validator.
 * 
 * Provides functions for validating DXC data against a system catalog.
 */
public class DataValidator {
    private SystemCatalog sc;

    public DataValidator(SystemCatalog sc) throws Exception {
        if (sc == null) {
            throw new Exception("DataValidator requires valid SystemCatalog.");
        }
        this.sc = sc;
    }

    public void validateData(DxcData d) throws Exception {
        if (d instanceof SensorData) {
            validateData((SensorData) d);
        } else if (d instanceof FaultInjectData) {
            validateData((FaultInjectData) d);
        } else if (d instanceof DiagnosisData) {
            validateData((DiagnosisData) d);
        }
    }

    /**
     * Validate SensorData
     */
    public void validateData(SensorData sd) throws Exception {
        Map<String, Value> svm = sd.getSensorValueMap();
        for (String s : svm.keySet()) {
            if (!sc.isComponentValid(s)) {
                throw new Exception("Invalid component ID in sensor message: "
                        + s + " at timestamp: " + sd.getTimeStamp());
            }
        }
    }

    /**
     * Validate FaultInjectData
     */
    public void validateData(FaultInjectData fid) throws Exception {
        String cid = fid.getFault().getComponentID();
        String mid = fid.getFault().getModeID();

        if (!sc.isComponentValid(cid)) {
            throw new Exception("Invalid component ID in fault: " + cid
                    + " at timestamp: " + fid.getTimeStamp());
        } else if (!sc.isFaultModeValid(cid, mid)) {
            throw new Exception("Invalid mode ID in fault: " + cid + " = "
                    + mid + " at timestamp: " + fid.getTimeStamp());
        }
    }

    /**
     * Validate DiagnosisData
     */
    public void validateData(DiagnosisData dd) throws Exception {
        for (Candidate c : dd.getCandidateSet()) {
            for (Fault f : c.getFaultSet()) {

                if (!sc.isComponentValid(f.getComponentID())) {
                    throw new Exception("Invalid component ID in diagnosis: "
                            + f.getComponentID() + " at timestamp: "
                            + dd.getTimeStamp());
                } else if (!sc.isFaultModeValid(f)) {
                    throw new Exception("Invalid mode ID in diagnosis: "
                            + f.getComponentID() + " = " + f.getModeID()
                            + " at timestamp: " + dd.getTimeStamp());
                }
            }
        }
    }

    /**
     * Strip component/mode map of invalid IDs.
     */
    public void stripInvalid(FaultSet faultSet) {
        FaultSet toRemove = new FaultSet();
        for (Fault f : faultSet) {
            if (!sc.isComponentValid(f.getComponentID())) {
                System.err.println("Unknown component ID in diagnosis: "
                        + f.getComponentID() + ".");
                toRemove.add(f);
            } else if (!sc.isFaultModeValid(f)) {
                System.err.println("Unknown failure mode ID in diagnosis: "
                        + f.getComponentID() + " = " + f.getModeID() + ".");
                toRemove.add(f);
            }
        }
        faultSet.removeAll(toRemove);
    }
}
