/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.example;

import org.dxc.api.DxcCallback;
import org.dxc.api.connection.ConnectorFactory;
import org.dxc.api.connection.DxcConnector;
import org.dxc.api.datatypes.*;
import java.util.Iterator;
import java.util.Map;

/**
 * ExampleJavaDA demonstrates how a diagnosis algorithm should send and receive
 * scenario data.
 */
public class ExampleJavaDA {

    private static boolean run = true;

    /**
     * This is the DA's implementation of the DxcCallback interface. After an
     * object of this class is registered with a DxcConnector, the connector
     * will execute its processData() method every time data is received.
     */
    @SuppressWarnings("unused")
    private static final DxcCallback exampleCallback = new DxcCallback() {
        public void processData(DxcData d) {
            /*
             * d is incoming data. Find out what type it is, downcast it to that
             * DxcData type, and process accordingly.
             */
            if (d instanceof SensorData) {
                // update observations

                SensorData sd = (SensorData) d;
                Map<String, Value> sensors = sd.getSensorValueMap();

                Iterator<String> i = sensors.keySet().iterator();

                while (i.hasNext()) {
                    String sensorID = i.next();
                    Value v = sensors.get(sensorID);

                    /*
                     * Here we don't know what datatype the value is, but this
                     * information can be looked up by sensor ID in the system's
                     * XML catalog
                     */
                    if (v instanceof RealValue) {
                        // cast to RealValue, retreive value
                        double rv = ((RealValue) v).get();

                        // write (sensorID, rv) to DA model/data structure for
                        // diagnosis
                    } else if (v instanceof BoolValue) {
                        // cast to BoolValue, et cetera...
                    }
                }

            } else if (d instanceof CommandData) {
                // update expected system state

                CommandData cd = (CommandData) d;
                for (Command c : cd.getCommands()) {
                    String commandID = cd.getCommandID();
                    Value v = cd.getValue();
                    // write command to model/data structure for diagnosis
                }
            } else if (d instanceof ScenarioStatusData) {
                // check for scenario end, exit if ended

                ScenarioStatusData stat = (ScenarioStatusData) d;
                if (stat.getStatus().equals(ScenarioStatusData.ENDED))
                    run = false;
            } else if (d instanceof ErrorData) {
                System.out.print("JavaDA received Error: ");
                System.out.print(((ErrorData) d).getError() + "\n");
            }
        }
    };

    private static DiagnosisData generateDiagnosis() {
        CandidateSet cs = new CandidateSet();
        Candidate c1 = new Candidate(), c2 = new Candidate(), c3 = new Candidate();

        /*
         * Here we populate candidates with made-up data. In reality component
         * IDs and mode IDs must match XML system descriptions.
         */
        c1.getFaultSet().add(new Fault("c", "stuckOpen"));
        c1.getFaultSet().add(new Fault("d", "stuckClosed"));
        c1.setWeight(0.2);

        c2.getFaultSet().add(new Fault("a", "bias"));
        c2.getFaultSet().add(new Fault("b", "switchedOff"));
        c2.setWeight(0.3);

        c3.getFaultSet().add(new Fault("c", "stuckClosed"));
        c3.getFaultSet().add(new Fault("d", "stuckOpen"));
        c3.setWeight(0.5);

        cs.add(c1);
        cs.add(c2);
        cs.add(c3);

        return new DiagnosisData(true, true, cs, "ExampleJavaDA notes.");
    }

    public static void main(String[] args) {
        try {
            /*
             * Create a DxcConnector. A scenario data source (SDS) must be
             * running in order to connect successfully.
             */
            DxcConnector c = ConnectorFactory.getDAConnector(exampleCallback);

            // signal to the system that we're prepared to receive data
            c.sendMessage(new ScenarioStatusData(ScenarioStatusData.DA_READY));

            /*
             * For this simple example, we'll create a diagnosis and send it
             * every 3 seconds from the main execution loop. Many DAs will
             * prefer to send diagnoses from the DxcCallback.processData()
             * method (above), after processing incoming data, or from a
             * separate thread dedicated to system state reasoning.
             */

            DiagnosisData d = generateDiagnosis();

            while (run) {
                // Send the same diagnosis every 3 seconds.
                c.sendMessage(d);
                Thread.sleep(3000);
            }

        } catch (Exception ex) {
            System.out.append(ex.toString() + " " + ex.getMessage());
        }

        System.exit(0);
    }

}
