package org.dxc.evaluator;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.dxc.evaluator.metrics.*;

public class DAResult implements Comparable<DAResult> {
    private String name;
    private String system;
    private Map<MetricInfo, DAMetricCalculator> metricCalcs;
    private List<ScenarioResult> scenarioResults;

    public DAResult(String DAName, String system) throws Exception {
        this.name = (DAName == null) ? "" : DAName;
        this.system = (system == null) ? "" : system;

        EvalConfig ec = EvalConfig.getInstance();
        metricCalcs = new HashMap<MetricInfo, DAMetricCalculator>();
        scenarioResults = new LinkedList<ScenarioResult>();

        for (MetricInfo m : ec.getMetrics()) {
            DAMetricCalculator dam = MetricCalculatorFactory.getDAMetric(m
                    .getDAMetricName());

            if (dam == null) {
                throw new Exception("Unknown metric: " + m.getName());
            } else {
                dam.setMinDataPoints(m.getMinDataPoints());
                metricCalcs.put(m, dam);
            }
        }
    }

    public String getDAName() {
        return name;
    }

    public String getSystem() {
        return system;
    }

    public void addScenario(ScenarioResult sr) throws Exception {
        for (MetricInfo m : metricCalcs.keySet()) {
            DAMetricCalculator dmc = metricCalcs.get(m);
            ScenarioMetricCalculator smc = sr.getCalculator(m
                    .getScenarioMetricName());
            if (smc != null)
                dmc.addSMCalculator(smc);
            else
                throw new Exception("Unknown scenario metric: \""
                        + m.getScenarioMetricName() + "\"");
        }
    }

    public double getMetricScore(MetricInfo m) {
        double ret = Double.NaN;
        DAMetricCalculator smc = metricCalcs.get(m);
        if (smc != null)
            ret = smc.getScore();

        return ret;
    }

    public int compareTo(DAResult d) {
        return this.getDAName().compareTo(d.getDAName());
    }

    public void calculate() {
        for (DAMetricCalculator dmc : metricCalcs.values()) {
            dmc.calculate();
        }

        scenarioResults.clear();
    }

    public void print() {
        System.out.println("DA: " + this.getDAName());
        System.out.println("System: " + this.getSystem());

        DecimalFormatSymbols dfs = new DecimalFormatSymbols();
        dfs.setNaN("[undefined]");
        DecimalFormat df = new DecimalFormat("0.0###", dfs);

        for (MetricInfo m : metricCalcs.keySet()) {
            System.out.println(m.getPrintPrefix()
                    + df.format(metricCalcs.get(m).getScore())
                    + m.getPrintPostfix());
        }
    }
}
