package org.dxc.evaluator.metrics;

import java.util.LinkedList;
import java.util.List;

import org.dxc.api.datatypes.Candidate;
import org.dxc.api.datatypes.DataValidator;
import org.dxc.api.datatypes.DxcData;
import org.dxc.api.datatypes.Fault;
import org.dxc.api.datatypes.FaultInjectData;
import org.dxc.api.datatypes.DiagnosisData;
import org.dxc.api.datatypes.FaultSet;
import org.dxc.api.datatypes.ScenarioData;
import org.dxc.api.datatypes.SystemCatalogXML;

/*
 * CPU Time metric.
 */
public class ClassificationErrors implements ScenarioMetricCalculator {
    private FaultSet faultSet;
    private DiagnosisData lastDiagnosis;
    private List<FaultInjectData> faults;
    private double errors;
    private DataValidator dv;

    ClassificationErrors() {
        faultSet = new FaultSet();
        faults = new LinkedList<FaultInjectData>();
        lastDiagnosis = null;
        errors = Double.NaN;
        dv = null;
    }

    public void addData(DxcData d) {
        if (d instanceof FaultInjectData) {
            FaultInjectData fid = (FaultInjectData) d;
            faults.add(fid);
            if (fid.isInjection()) {
                faultSet.add(fid.getFault());
            } else {
                faultSet.remove(fid.getFault());
            }
        } else if (d instanceof DiagnosisData) {
            DiagnosisData dd = (DiagnosisData) d;
            if (lastDiagnosis != null) {
                if (!lastDiagnosis.getCandidateSet().equals(
                        dd.getCandidateSet())) {
                    lastDiagnosis = dd;
                }
            } else if (dd.getCandidateSet().size() > 0) {
                lastDiagnosis = dd;
            }
        } else if (d instanceof ScenarioData) {
            ScenarioData sd = (ScenarioData) d;
            try {
                dv = new DataValidator(SystemCatalogXML.getInstance(sd
                        .getSystemName()));
            } catch (Exception ex) {
                System.err.println("Unable to load catalog for system "
                        + sd.getSystemName());
                dv = null;
            }
        }
    }

    public void calculate() {
        DiagnosisData dd = lastDiagnosis;
        FaultSet injected = new FaultSet();

        for (Fault f : faultSet) {
            injected.add(new Fault(f.getComponentID(), f.getModeID()));
        }

        if ((dd == null || dd.getCandidateSet() == null || dd.getCandidateSet()
                .size() == 0)) {
            if (injected.size() > 0) {
                if (((Double) errors).isNaN())
                    errors = injected.size();
                else
                    errors += injected.size();
            }
        }

        if (dd == null) {
            dd = new DiagnosisData(false, false, null, null);
        }

        for (Candidate c : dd.getCandidateSet()) {
            FaultSet diagnosis = new FaultSet();

            for (Fault f : c.getFaultSet()) {
                diagnosis.add(new Fault(f.getComponentID(), f.getModeID()));
            }

            if (dv != null) {
                dv.stripInvalid(diagnosis);
            }

            FaultSet unionDI = new FaultSet(diagnosis);
            unionDI.addAll(injected);

            if (unionDI.size() == 0)
                return;

            // mismatches = symmetricDifference(faults, hypothesis)
            FaultSet mismatches = new FaultSet(unionDI);
            diagnosis.retainAll(injected); // intersection
            mismatches.removeAll(diagnosis); // subtraction

            double w = c.getWeight();
            double numMismatch = mismatches.size();

            if (((Double) errors).isNaN())
                errors = numMismatch * w;
            else
                errors += numMismatch * w;
        }
    }

    public double getScore() {
        return errors;
    }

    public void print() {
        if (lastDiagnosis != null)
            lastDiagnosis.put(System.out);

        if (!faults.isEmpty())
            for (FaultInjectData f : faults)
                f.put(System.out);

        if (!((Double) errors).isNaN()) {
            System.out.println("Classification Errors: " + errors);
        }
    }
}