package org.dxc.evaluator.metrics;

import java.util.LinkedList;
import java.util.List;

import java.util.Iterator;

import org.dxc.api.datatypes.DxcData;
import org.dxc.api.datatypes.FaultInjectData;
import org.dxc.api.datatypes.DiagnosisData;
import org.dxc.api.datatypes.ScenarioStatusData;
import org.dxc.api.datatypes.ErrorData;

/*
 * CPU Time metric.
 */
public class Detection implements ScenarioMetricCalculator {
    public enum DetectionType {
        FALSE_NEGATIVE, FALSE_POSITIVE, TRUE_NEGATIVE, TRUE_POSITIVE, UNDEFINED
    };

    private DetectionType detection;
    private List<FaultInjectData> faults;
    private List<DiagnosisData> diags;
    private long startTime;
    private double detectionTime;
    private boolean faultScenario;

    Detection() {
        detection = DetectionType.UNDEFINED;
        faults = new LinkedList<FaultInjectData>();
        diags = new LinkedList<DiagnosisData>();
        startTime = -1;
        detectionTime = Double.NaN;
        faultScenario = false;
    }

    public DetectionType getDetection() {
        return detection;
    }

    public double getDetectionTime() {
        return detectionTime;
    }

    public boolean isFaultScenario() {
        return faultScenario;
    }

    public void addData(DxcData d) {
        if (startTime < 0
                && !(d instanceof ScenarioStatusData
                        || d instanceof DiagnosisData || d instanceof ErrorData)) {
            startTime = d.getTimeStamp();
        }

        if (d instanceof FaultInjectData) {
            faults.add((FaultInjectData) d);
        } else if (d instanceof DiagnosisData) {
            diags.add((DiagnosisData) d);
        }
    }

    public void calculate() {
        long injectionTime = -1, postInjTime = -1, detTime = -1;

        if (startTime < 0) {
            detection = DetectionType.UNDEFINED;
            return;
        }

        if (!faults.isEmpty()) {
            // assumes all faultInject messages are injections
            injectionTime = faults.get(0).getTimeStamp();
            faultScenario = true;

            if (postInjTime < 0) {
                postInjTime = injectionTime;
            }
        }

        Iterator<DiagnosisData> ddi = diags.iterator();
        boolean detected = false;

        while (ddi.hasNext()) {
            DiagnosisData diag = ddi.next();
            if (diag.getDetectionSignal()) {
                detTime = diag.getTimeStamp();
                detected = true;
                break;
            }
        }
        /*
         * uncomment to take detection signal from _last_ diagnosis if
         * (!dds.isEmpty()) detection =
         * dds.get(dds.size()-1).getDetectionSignal();
         */

        if (detected) {
            if (injectionTime == -1 // <- no faults.
                    || detTime < postInjTime) { // <- detected early.
                this.detection = DetectionType.FALSE_POSITIVE;
            } else {
                detection = DetectionType.TRUE_POSITIVE;
                detTime -= injectionTime;
                detectionTime = detTime;
            }
        } else {
            if (injectionTime == -1) {
                detection = DetectionType.TRUE_NEGATIVE;
            } else {
                detection = DetectionType.FALSE_NEGATIVE;
            }
        }
    }

    public double getScore() {
        return this.getDetectionTime();
    }

    public void print() {
        System.out.print("Detection: ");
        switch (this.getDetection()) {
        case FALSE_POSITIVE:
            System.out.println("False Positive.");
            break;
        case FALSE_NEGATIVE:
            System.out.println("False Negative.");
            break;
        case TRUE_NEGATIVE:
            System.out.println("True Negative.");
            break;
        case TRUE_POSITIVE:
            System.out.println("True Positive.");
            break;
        default:
            System.out.println("Unknown.");
            break;
        }

        System.out.println("Detection time: " + this.getDetectionTime());
    }
}