package org.dxc.evaluator.metrics;

import org.dxc.api.DxcCallback;
import org.dxc.api.datatypes.*;

public class RecoveryCost implements ScenarioMetricCalculator {
    private double score = Double.NaN;
    private CommandSet daCommands = new CommandSet();
    private FaultSet faults = new FaultSet();
    private OracleInterface oi;
    private long startTime = -1;

    private class RcCallback extends DxcCallback {
        private RecoveryData rd = null;

        @Override
        public void processData(DxcData data) {
            if (data instanceof RecoveryData) {
                rd = (RecoveryData) data;
                synchronized (this) {
                    this.notifyAll();
                }
            }
        }

        public RecoveryData getRecoveryData() {
            return rd;
        }
    }

    private RcCallback rcCallback = new RcCallback();

    public RecoveryCost() {
        oi = OracleInterface.getInstance(rcCallback);
    }

    public void addData(DxcData d) {
        if (d instanceof ScenarioData) {
            oi.getConnector().sendMessage(d);
        } else if (d instanceof CommandData) {
            if (d.getSource() != null && d.getSource().equals("SDS")) {
                oi.getConnector().sendMessage(d);
            } else {
                daCommands.addAll(((CommandData) d).getCommands());
            }
        } else if (d instanceof FaultInjectData) {
            Fault f = ((FaultInjectData) d).getFault();
            StringValueMap fpm = f.getParameterMap();

            // workaround for StringValue parameters
            for (String s : fpm.keySet()) {
                Value v = fpm.get(s);
                if (v instanceof StringValue) {
                    String sv = ((StringValue) v).get();
                    v = new RealValue(Double.valueOf(sv));
                    fpm.put(s, v);
                }
            }
                
            if (startTime >= 0 && !fpm.containsKey("FaultTime"))
                fpm.put("FaultTime",
                        new RealValue(d.getTimeStamp() - startTime));

            faults.add(f);
        } else if (d instanceof ScenarioStatusData) {
            ScenarioStatusData ssd = (ScenarioStatusData) d;

            if (ssd.getStatus().equals(ScenarioStatusData.STARTED))
                startTime = ssd.getTimeStamp();

            if (!ssd.getStatus().equals(ScenarioStatusData.DA_ENDED))
                oi.getConnector().sendMessage(d);
        }
    }

    public void calculate() {
        try {
            if (daCommands.isEmpty())
                daCommands.add(new Command("systemAction", new StringValue(
                        "NOP")));

            RecoveryData rd = new RecoveryData(faults, daCommands);

            synchronized (rcCallback) {
                oi.getConnector().sendMessage(rd);
                rcCallback.wait(10000);
            }

            rd = rcCallback.getRecoveryData();

            if (rd != null)
                score = rd.getCost();
            else
                throw new Exception("Oracle returned no message.");

            OracleInterface.clearOutput();

        } catch (Exception ex) {
            System.out.println(ex);
            System.out.println("Restarting Oracle.");
            try {
                OracleInterface.shutdown();
                Thread.sleep(1000);
                oi = OracleInterface.getInstance(rcCallback);
            } catch (Exception ignore) {
            }
        }

    }

    public double getScore() {
        return score;
    }

    public void print() {
        System.out.println("Recovery Cost: " + score);
    }

}
